<?php

/**
 * This is the model class for table "audioFile".
 *
 * The followings are the available columns in table 'audioFile':
 * @property integer $id
 * @property string $name
 * @property string $type
 */
class AudioFileUseCase
{

    const QUEUE = 1;
    const CONFERENCE = 2;
    const FORWARDING = 4;
    const IVR = 8;
    const CALL_WAITING = 16;

    public static function compare($useCase, $flag)
    {
        if ($useCase & $flag) {
            return true;
        }
        return false;
    }

    public static function allUseCases()
    {
        $cases = array();

        if (PhonalisaLicenseManager::routingQueuesMax()) {
            $cases['QUEUE'] = AudioFileUseCase::QUEUE;
        }

        if (PhonalisaLicenseManager::routingConferenceMax()) {
            $cases['CONFERENCE'] = AudioFileUseCase::CONFERENCE;
        }

        $cases['FORWARDING'] = AudioFileUseCase::FORWARDING;

        if (PhonalisaLicenseManager::routingIVRMax()) {
            $cases['IVR'] = AudioFileUseCase::IVR;
        }

        $cases['CALL_WAITING'] = AudioFileUseCase::CALL_WAITING;

        // FIXME add translation 
        return $cases;
    }
}

class AudioFileType
{

    const privateType = 'private';
    const systemType = 'system';
    const musicOnHold = 'moh';
    const alarmType = 'alarm';

    public static function getAudioFileTypesAsString($type)
    {
        switch ($type) {
            case AudioFileType::privateType :
                $type = __('AUDIOFILES_MODEL_TYPE_PRIVATE');
                break;
            case AudioFileType::systemType :
                $type = __('AUDIOFILES_MODEL_TYPE_SYSTEM');
                break;
            case AudioFileType::musicOnHold :
                $type = __('AUDIOFILES_MODEL_TYPE_MOH');
                break;
            default :
                $type = 'undefined';
        }
        return $type;
    }
}

class AudioFile extends PhonalisaRecord
{

    public $id;
    public $name;
    public $type;
    public $descr;
    public $duration;
    public $created;
    public $user_id;
    public $data;
    public $playlistID;
    public $fileName;
    public $usecase;

    public static function myAttributes()
    {
        return array(
            'name' => 'name',
            'descr' => 'descr',
            'duration' => 'duration',
            'created' => array('key' => 'created', 'type' => 'dateTime'),
            'type' => 'type',
            'user_id' => 'user_id',
            'data' => 'data',
            'playlistID' => 'playlistID',
            'fileName' => 'fileName',
            'usecase' => 'usecase',
        );
    }

    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return Gateway the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * @return string the associated database table name
     */
    public function tableName()
    {
        return 'audioFile';
    }

    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
            array('fileName, name, usecase', 'safe'), //FIXME !
            array('type, descr, user_id, duration, created, playlistID', 'length', 'max' => 64),
            // The following rule is used by search().
            // Please remove those attributes that should not be searched.
            array('id, name, type, descr, duration, created, data, playlistID', 'safe', 'on' => 'search'),
            array(
                'data',
                'ext.PhonalisaValidators.fileValidator',
                'allowEmpty' => false,
                'mimeTypes' => array('audio/x-wav', 'audio/wav'),
                'fileInfoRaw' => 'RIFF (little-endian) data, WAVE audio, Microsoft PCM, 16 bit, mono 22050 Hz',
                'on' => 'create',
                'message' => __('AUDIOFILES_ERROR_MESSAGE_NO_VALID_AUDIO_FORMAT')
            ),
            array(
                'data',
                'ext.PhonalisaValidators.fileValidator',
                'allowEmpty' => true,
                'mimeTypes' => array('audio/x-wav', 'audio/wav'),
                'fileInfoRaw' => 'RIFF (little-endian) data, WAVE audio, Microsoft PCM, 16 bit, mono 22050 Hz',
                'message' => __('AUDIOFILES_ERROR_MESSAGE_NO_VALID_AUDIO_FORMAT')
            ),
        );
    }

    /**
     * @return array relational rules.
     */
    public function relations()
    {
        // NOTE: you may need to adjust the relation name and the related
        // class name for the relations automatically generated below.
        return array(
        );
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'id' => __('AUDIOFILES_MODEL_ID'),
            'name' => __('AUDIOFILES_MODEL_NAME'),
            'descr' => __('AUDIOFILES_MODEL_DESCR'),
            'duration' => __('AUDIOFILES_MODEL_DURATION'),
            'created' => __('AUDIOFILES_MODEL_CREATED'),
            'user_id' => __('AUDIOFILES_MODEL_USER_ID'),
            'data' => __('AUDIOFILES_MODEL_DATA'),
            'type' => __('AUDIOFILES_MODEL_TYPE'),
            'playlistID' => __('AUDIOFILES_MODEL_PLAYLIST_ID'),
            'fileName' => __('AUDIOFILES_MODEL_FILE_NAME'),
            'usecase' => __('AUDIOFILES_MODEL_USECASE'),
        );
    }

    /**
     * Retrieves a list of models based on the current search/filter conditions.
     * @return CPhonalisaDataProvider the data provider that can return the models
     * based on the search/filter conditions.
     */
    public function search($order = 'name', $playlistID = null)
    {
        // Warning: Please modify the following code to remove attributes that
        // should not be searched.
        $criteria = new CDbCriteria;
        if ($playlistID != null) {
            $criteria->condition = 'type=moh AND playlistID=' . $playlistID;
        }

        $filter = $this->getFilter($criteria, $order);
        if ($filter != null) {
            return new CPhonalisaDataProvider($this, $filter);
        }
    }

    public function searchPrivateAudiofiles($order = 'name')
    {
        $criteria = new CDbCriteria;
        $criteria->condition = 'type=' . AudioFileType::privateType . ' AND user_id=' . Yii::app()->user->sudoUser;
        $filter = $this->getFilter($criteria, !isset($_REQUEST['sort']) ? $order : $_REQUEST['sort']);
        if ($filter != null) {
            return new CPhonalisaDataProvider($this, $filter);
        }
    }

    public function searchSystemAudiofiles($order = 'name')
    {
        $criteria = new CDbCriteria;
        $criteria->condition = 'type=' . AudioFileType::systemType;
        $filter = $this->getFilter($criteria, $order);

        $filter['pagination'] = array(
            'pageSize' => getGridPageSize('audio-file-grid'),
        );

        if ($filter != null) {
            return new CPhonalisaDataProvider($this, $filter);
        }
    }

    public function beforeSave()
    {
        // for systemMusicOnHold module
        if (isset($_FILES['AudioFile']) && $_FILES['AudioFile']['name']["data"] != '') {
            $this->fileName = $_FILES['AudioFile']['name']["data"];
        }

        if (isset($_POST['AudioFile']['data']) && $_POST['AudioFile']['data'] == '' && !empty($_POST['recordId'])) {
            $this->fileName = 'record';
        }

        //for avisoAPI
        if (isset($_FILES['uploadaudio']) && $_FILES['uploadaudio']['name'] != '') {
            $this->fileName = $_FILES['uploadaudio']['name'];
        }

        if (trim($this->name) == '') {
            $fileName = pathinfo($this->fileName);
            $this->name = $fileName['filename'];
        }

        if ($this->isNewRecord()) {
            $this->created = PhonalisaDate::newServerDate();
        }

        $this->user_id = Yii::app()->user->sudoUser;

        return parent::beforeSave();
    }

    public function afterSave()
    {
        //for systemMusicOnHold module : 
        if (!empty($_FILES['AudioFile']['tmp_name']["data"])) {
            if ($_FILES['AudioFile']['tmp_name']["data"] != '') {
                $this->saveFileToFieldByPath("data", $_FILES['AudioFile']['tmp_name']["data"]);
            }
        }

        //for avisoAPI
        if (!empty($_FILES['uploadaudio']['tmp_name'])) {
            if ($_FILES['uploadaudio']['tmp_name'] != '') {
                $this->saveFileToFieldByPath("data", $_FILES['uploadaudio']['tmp_name']);
            }
        }

        return parent::afterSave();
    }

    public function getAudioFileList($usecase = null)
    {
        $models = AudioFile::model()->find(array('type' => 'system'), 'name');
        if ($usecase !== null) {
            foreach ($models as $key => $model) {
                if (!AudioFileUseCase::compare($model->usecase, $usecase)) {
                    unset($models[$key]);
                }
            }
        }
        $list = array();

        foreach ($models as $audioFile) {
            $list[$audioFile->id] = $audioFile->name;
        }
        $list = array('' => __('AUDIOFILE_MODEL_SELECT_VOICEMESSAGE')) + $list;
        return $list;
    }

    public function getFormattedCreatedDate($data)
    {
        return PhonalisaDate::userDateTime($data->created);
    }

    public function getUseCaseDataprovider()
    {
        $data = array();
        $id = 0;
        foreach (AudioFileUseCase::allUseCases() as $name => $flag) {
            $data[] = array(
                'id' => $id++,
                'name' => $name,
                "flag" => $flag,
                "translation" => __("AUDIOFILE_MODEL_USECASE_$name"),
                'currentUseCase' => $this->usecase,
                'isNewRecord' => $this->isNewRecord(),
            );
        }
        return new CArrayDataProvider($data);
    }

    public function validateUploadedData()
    {
        $file = $_FILES['AudioFile'];
        $this->data = new CUploadedFile($file['name']["data"], $file["tmp_name"]["data"], $file["type"]["data"], $file["size"]["data"], $file["error"]["data"]);
        $this->validate("data");
        if (isset($this->errors["data"])) {
            if (is_array($this->errors["data"])) {
                throw new CHttpException(500, $this->errors["data"][0]);
            } else {
                throw new CHttpException(500, $this->errors["data"]);
            }
        }
    }
}
